﻿using System.Web.UI;
using System.Web.UI.WebControls;
using EPiServer.Cms.AddOns.Blocks;
using EPiServer.Cms.AddOns.Blocks.Web.Controls;
using EPiServer.Core;
using EPiServer.Framework.DataAnnotations;
using EPiServer.Web;
using EPiServer.Web.WebControls;

namespace EPiServer.Templates.Alloy.Business.WebControls
{
    /// <summary>
    /// A Renderer for the Optimzing Block also implementing the <see cref="IBlockControl"/> interface
    /// to enable the correct layout and style when previewing.
    /// </summary>
    [TemplateDescriptor(ModelType = typeof(OptimizingBlock), Default = true, AvailableWithoutTag = true, Inherited = true)]
    public class OptimzingBlockControl : OptimizingBlockControl, IBlockControl
    {
        private IBlockControl _renderer;

        /// <summary>
        /// Creates the renderer.
        /// </summary>
        private IBlockControl ResolveRenderer()
        {
            Control control = null;
            IContent currentData = CurrentData as IContent;

            // Did we get a variation?
            if (currentData != base.CurrentData) 
            {
                control = CreateRenderer(currentData);
            } 
            else 
            {
                control = new Literal();
            }

            // If the loaded control doesn't implement the correct interface we need to wrap it. 
            if (!(control is IBlockControl))
            {
                var wrapper = new GenericIBlockControlWrapper
                {
                    InnerControl = control,
                    CurrentData = currentData
                };

                wrapper.EnsureChildControlsCreated();
                control = wrapper;
            }

            return control as IBlockControl;
        }

        /// <summary>
        /// Adds the renderer. Called by CreateChildControls.
        /// </summary>
        protected override void AddRenderer()
        {
            Controls.Add((Control)Renderer);
        }

        /// <summary>
        /// Gets the control responsible for rendering the selected variation.
        /// </summary>
        /// <value>The renderer.</value>
        protected virtual IBlockControl Renderer
        {
            get { return _renderer ?? (_renderer = ResolveRenderer()); }
        }

        /// <summary>
        /// Gets or sets the CSS class.
        /// </summary>
        /// <value>The CSS class.</value>
        public string CssClass
        {
            get
            {
                var control = Renderer as WebControl;
                return control != null ? control.CssClass : string.Empty;
            }
            set
            {
                var control = Renderer as WebControl;

                if (control != null)
                {
                    control.CssClass = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets a CSS class for this block's rendering container
        /// </summary>
        public virtual string ContainerCssClass
        {
            get { return Renderer.ContainerCssClass; }
            set { Renderer.ContainerCssClass = value; }
        }

        /// <summary>
        /// Gets or sets the minimum width required by the block control to aid in block rendering
        /// </summary>
        public virtual int MinimumWidth
        {
            get { return Renderer.MinimumWidth; }
            set { Renderer.MinimumWidth = value; }
        }

        /// <summary>
        /// Gets or sets the maximum width offered by the block control to aid in block rendering
        /// </summary>
        public virtual int MaximumWidth
        {
            get { return Renderer.MaximumWidth; }
            set { Renderer.MaximumWidth = value; }
        }

        /// <summary>
        /// Gets or sets the width of which to render the block control, for example 4 for "span4"
        /// </summary>
        public int Width
        {
            get { return Renderer.Width; }
            set { Renderer.Width = value; }
        }

        /// <summary>
        /// Gets or sets the current data.
        /// </summary>
        /// <value>The current data.</value>
        public new IContentData CurrentData
        {
            get
            {
                if (base.CurrentData != null)
                {
                    // Need to return the selected variation to get the correct css class on the container
                    return (IContentData)ResolveVariation(base.CurrentData) ?? base.CurrentData;
                }

                return base.CurrentData;
            }
            set { base.CurrentData = (OptimizingBlock)value; }
        }

        /// <summary>
        /// Resolves the selected variation, drilling down multiple levels of OptimizingBlocks if necessary.
        /// </summary>
        private IContent ResolveVariation(OptimizingBlock optimizingBlock)
        {
            IContent content = optimizingBlock as IContent;

            while (content is OptimizingBlock)
            {
                content = ((OptimizingBlock)content).GetSelectedVariation();
            }

            return content;
        }
    }
}